/*
 * @video_stage.c
 * @author marc-olivier.dzeukou@parrot.com
 * @date 2007/07/27
 *
 * ihm vision thread implementation
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <termios.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>

#include <gtk/gtk.h>
#include <sys/time.h>
#include <time.h>

#include <VP_Api/vp_api.h>
#include <VP_Api/vp_api_error.h>
#include <VP_Api/vp_api_stage.h>
#include <VP_Api/vp_api_picture.h>
#include <VP_Stages/vp_stages_io_file.h>
#include <VP_Stages/vp_stages_i_camif.h>

#include <config.h>
#include <VP_Os/vp_os_print.h>
#include <VP_Os/vp_os_malloc.h>
#include <VP_Os/vp_os_delay.h>
#include <VP_Stages/vp_stages_yuv2rgb.h>
#include <VP_Stages/vp_stages_buffer_to_picture.h>
#include <VLIB/Stages/vlib_stage_decode.h>

#include <ardrone_tool/ardrone_tool.h>
#include <ardrone_tool/Com/config_com.h>

#ifndef RECORD_VIDEO
#define RECORD_VIDEO
#endif
#ifdef RECORD_VIDEO
#    include <ardrone_tool/Video/video_stage_recorder.h>
#endif

#include <ardrone_tool/Video/video_com_stage.h>

#include "Video/video_stage.h"

#define NB_STAGES 10

PIPELINE_HANDLE pipeline_handle;

uint8_t*  pixbuf_data_real       = NULL;
uint8_t*  pixbuf_data_robe       = NULL;
uint8_t*  pixbuf_data_sword      = NULL;

vp_os_mutex_t  video_update_lock = PTHREAD_MUTEX_INITIALIZER;

C_RESULT output_gtk_stage_open( void *cfg, vp_api_io_data_t *in, vp_api_io_data_t *out)
{
    return (SUCCESS);
}

/*defines, includes and other stuff for vision*/

#include <cv.h>
#include <highgui.h>

//ROBE

extern int MINHROBE;
extern int MAXHROBE;
extern int MINSROBE;
extern int MAXSROBE;
extern int MINVROBE;
extern int MAXVROBE;

//SWORD

extern int MINHSWORDL;
extern int MAXHSWORDL;
extern int MINHSWORDH;
extern int MAXHSWORDH;
extern int MINSSWORD;
extern int MAXSSWORD;
extern int MINVSWORD;
extern int MAXVSWORD;

//ImageData
extern int buffWidth;
extern int buffHeight;

extern int itemHeightInCm;

extern int minAllowedPixelHeight;
extern int DEFAULTDISTANCEINCM;
//210 is a magic number that comes out from the point of focus of the camera!
extern int pointOfFocus;

//Wieghted Mean
extern float PASTWEIGHT_DIST;

//global variables for variables that should be passed in the function ma che schifo
extern int distanceInCmParameter;
extern int pixelsFromCenterParameter;
extern float chancheOfShootParameter;
extern int lookaround;

extern int shotevaluationrequest;
extern int shotevaluationresponse;

void distanceFromTargetInCm(uchar * imageBuffer, int* distanceInCmParameter, int* pixelsFromCenterParameter, float* chancheOfShootParameter)
{
    //PHASE 1: DATA SETTINGS

    IplImage* buffer=cvCreateImageHeader(cvSize(buffWidth,buffHeight),8,3);
    //NOTE: by using cvSetData, the cvReleaseImage won't deallocate the actual imageData buffer because this is considered "external"  cvSetData(imageHeader,dataArray(void*),step);
    //cvSetData(buffer,(void*)imageBuffer,320*3);
    buffer->imageData=imageBuffer;
    IplImage* hsv=cvCreateImage(cvGetSize(buffer),8,3);
    cvCvtColor(buffer,hsv,CV_RGB2HSV);

    //PHASE 2: ALPHA SUPPORT IMAGES

    int height=hsv->height;
    int width=hsv->width;
    int stephsv=hsv->widthStep;
    int channels=hsv->nChannels;
    uchar* data=(uchar*)hsv->imageData;

    IplImage* colorAlphaRobe=cvCreateImage(cvGetSize(hsv),8,1);
    IplImage* colorAlphaSword=cvCreateImage(cvGetSize(hsv),8,1);

    uchar* dataColorAlphaRobe=(uchar*)colorAlphaRobe->imageData;
    uchar* dataColorAlphaSword=(uchar*)colorAlphaSword->imageData;

    int stepColorAlphaRobe=colorAlphaRobe->widthStep;
    int stepColorAlphaSword=colorAlphaSword->widthStep;

    int hValue,vValue,sValue;

    int i,j;

    for(i=0; i<height; i++)
    {
        for (j=0; j<width; j++)
        {
            //H is edited that way because of the way openCV stores it in 8-bits
            hValue=(data[i*stephsv+j*channels]*2*255)/360;
            sValue=data[i*stephsv+j*channels+1];
            vValue=data[i*stephsv+j*channels+2];

            //NOTE: Due to the circular nature of Hue, when looking for RED color, you need to edit the test to comprehend also the next 360 degrees.

            //vValue takes out the too dark colors, sValue the too light ones.
            dataColorAlphaRobe[i*stepColorAlphaRobe+j]=((hValue>MINHROBE)&&(hValue<MAXHROBE)&&(vValue>MINVROBE)&&(vValue<MAXVROBE)&&(sValue>MINSROBE)&&(sValue<MAXSROBE)) ? 255:0;

            dataColorAlphaSword[i*stepColorAlphaSword+j]=((((hValue>MINHSWORDL)&&(hValue<MAXHSWORDL))||((hValue>MINHSWORDH)&&(hValue<MAXHSWORDH)))&&(vValue>MINVSWORD)&&(vValue<MAXVSWORD)&&(sValue>MINSSWORD)&&(sValue<MAXSSWORD)) ? 255:0;
        }
    }

    cvReleaseImage(&hsv);

    //PHASE 3: CONTOUR ENANCHMENT

    int iterations=10;


    cvMorphologyEx(colorAlphaRobe,colorAlphaRobe,NULL,NULL,CV_MOP_CLOSE,iterations);
    cvMorphologyEx(colorAlphaSword,colorAlphaSword,NULL,NULL,CV_MOP_CLOSE,iterations);

    //PHASE 6: CONTOUR RELEVATION

    CvMemStorage* storage=cvCreateMemStorage(0);
    int headerSize=sizeof(CvContour);
    CvContourRetrievalMode mode=CV_RETR_LIST;
    CvChainApproxMethod method=CV_CHAIN_APPROX_TC89_L1;

    CvSeq* firstContourRobe=NULL;
    CvSeq* firstContourSword=NULL;

    IplImage *contourImageRobe=cvCreateImage(cvGetSize(colorAlphaRobe),8,1);
    cvCopy(colorAlphaRobe,contourImageRobe,NULL);

    IplImage *contourImageSword=cvCreateImage(cvGetSize(colorAlphaSword),8,1);
    cvCopy(colorAlphaSword,contourImageSword,NULL);

    int numberContoursRobe=cvFindContours(contourImageRobe,storage,&firstContourRobe,headerSize,mode,method,cvPoint(0,0));
    int numberContoursSword=cvFindContours(contourImageSword,storage,&firstContourSword,headerSize,mode,method,cvPoint(0,0));

    cvReleaseImage(&contourImageRobe);
    cvReleaseImage(&contourImageSword);

    //PHASE 6: MAX SIZE CONTOUR
    //ROBE

    CvSeq* contourRobe=firstContourRobe;
    CvSeq* maxContourRobe=NULL;
    CvRect maxBoxRobe=cvRect(0,0,0,0);
    CvRect boxRobe=cvRect(0,0,0,0);
    int maxAreaRobe=0;
    int areaRobe=0;
    for (i=0; i<numberContoursRobe; i++)
    {
        areaRobe = fabs(cvContourArea(contourRobe, CV_WHOLE_SEQ,0));
        boxRobe=cvBoundingRect(contourRobe,1);

        if (areaRobe>maxAreaRobe)
        {
            maxContourRobe=contourRobe;
            maxAreaRobe=areaRobe;
            maxBoxRobe=boxRobe;
        }
        contourRobe = contourRobe->h_next;
    }

    //SWORD

    CvSeq* contourSword=firstContourSword;
    CvSeq* maxContourSword=NULL;
    CvRect maxBoxSword=cvRect(0,0,0,0);
    CvRect boxSword=cvRect(0,0,0,0);

    int maxAreaSword=0;
    int areaSword=0;

    for (i=0; i<numberContoursSword; i++)
    {
        areaSword = fabs(cvContourArea(contourSword, CV_WHOLE_SEQ,0));
        boxSword=cvBoundingRect(contourSword,1);

        if (areaSword>maxAreaSword)
        {
            maxContourSword=contourSword;
            maxAreaSword=areaSword;
            maxBoxSword=boxSword;
        }
        contourSword = contourSword->h_next;
    }

    long counter=0;
    for(i=maxBoxRobe.y; i<=maxBoxRobe.y+maxBoxRobe.height; i++)
    {
        for (j=maxBoxRobe.x; j<=maxBoxRobe.x+maxBoxRobe.width; j++)
        {
            counter+=(dataColorAlphaSword[i*stepColorAlphaSword+j]) ? 1 : 0;
        }
    }


    cvReleaseImage(&buffer);
    cvReleaseImage(&colorAlphaRobe);
    cvReleaseImage(&colorAlphaSword);

    //At this point, counter stores the number of pixels belonging both to the robe and the sword (overlapping pixels),
    // so we calculate the chancheOf/ as 1 - the ratio between the pixels of the sword covering the robe and half of
    //the sword surface so to consider the perspective of the drone height (less than half sword is over the head)
    float tempValue=((maxBoxSword.height>0)&&(maxBoxSword.width>0))?(1.5*(float)counter/((float)maxBoxSword.width*(float)maxBoxSword.height)):0;
    tempValue=(tempValue>1)?1:tempValue;
    *chancheOfShootParameter=(1 - tempValue)*(0.2)+(*chancheOfShootParameter)*0.8;

    int latoQuadratoPixel=maxBoxRobe.width/3;
    if (shotevaluationrequest>0)
    {
        if (shotevaluationrequest>1)
        {
            shotevaluationrequest--;
        }
        else
        {

            int countParry=0;
            int r,c;
            for (r=0; r<latoQuadratoPixel; r++)
            {
                for (c=0; c<latoQuadratoPixel; c++)
                {
                    countParry+= (dataColorAlphaSword[((maxBoxRobe.y+maxBoxRobe.height/5)-(latoQuadratoPixel/2)+r)*stepColorAlphaSword+(maxBoxRobe.x+maxBoxRobe.width/2)-(latoQuadratoPixel/2)+c])? 1:0;
                }
            }
            shotevaluationrequest=0;
            if (countParry>=latoQuadratoPixel*latoQuadratoPixel/8)
            {
                shotevaluationresponse=1;//SHOT_PARRIED
            }
            else
            {
                shotevaluationresponse=2;//SHOT_PASSED
            }
        }
    }
    //Colouring Stream Alpha
    int vWhite=0;
    for(i=0; i<height; i++)
    {
        for (j=0; j<width; j++)
        {
            pixbuf_data_robe[vWhite]=dataColorAlphaRobe[i*stepColorAlphaRobe+j];
            pixbuf_data_robe[vWhite+1]=dataColorAlphaRobe[i*stepColorAlphaRobe+j];
            pixbuf_data_robe[vWhite+2]=dataColorAlphaRobe[i*stepColorAlphaRobe+j];

            vWhite+=3;
        }
    }


    //PHASE 8: DISTANCE ESTIMATION
    //Distance
    if ((maxBoxRobe.height>minAllowedPixelHeight)&&((pointOfFocus*itemHeightInCm/maxBoxRobe.height)<500))
    {
        //Object in sight, update parameters. Else do nothing, impose hovering
        *distanceInCmParameter=(*distanceInCmParameter)*PASTWEIGHT_DIST + (1 - PASTWEIGHT_DIST)*(pointOfFocus*itemHeightInCm/maxBoxRobe.height);
        *pixelsFromCenterParameter=(maxBoxRobe.x) + (maxBoxRobe.width/2)-(buffWidth/2);
        lookaround=0;
        //Colouring Box Center
        int vColors=0;
        for(i=0; i<height; i++)
        {
            for (j=0; j<width; j++)
            {


                //Sword
                if (dataColorAlphaSword[i*stepColorAlphaSword+j])
                {
                    pixbuf_data_sword[vColors]=255;
                    pixbuf_data_sword[vColors+1]=0;
                    pixbuf_data_sword[vColors+2]=0;
                }
                else
                {


                    if((maxBoxRobe.y<=i)&&(i<=maxBoxRobe.y+maxBoxRobe.height)&&(maxBoxRobe.x<=j)&&(j<=maxBoxRobe.x+maxBoxRobe.width))
                    {
                        if (((maxBoxRobe.y+maxBoxRobe.height/5)-(latoQuadratoPixel/2)<=i)&&((maxBoxRobe.y+maxBoxRobe.height/5)+(latoQuadratoPixel/2)>=i)&&((maxBoxRobe.x+maxBoxRobe.width/2)-(latoQuadratoPixel/2)<=j)&&((maxBoxRobe.x+maxBoxRobe.width/2)+(latoQuadratoPixel/2)>=j))
                        {
                            //Heart
                            pixbuf_data_sword[vColors]=0;
                            pixbuf_data_sword[vColors+1]=255;
                            pixbuf_data_sword[vColors+2]=0;
                        }
                        else
                        {
                            //Robe
                            pixbuf_data_sword[vColors]=0;
                            pixbuf_data_sword[vColors+1]=0;
                            pixbuf_data_sword[vColors+2]=255;
                        }
                    }

                    else
                    {
                        pixbuf_data_sword[vColors]=0;
                        pixbuf_data_sword[vColors+1]=0;
                        pixbuf_data_sword[vColors+2]=0;
                    }
                }


                vColors+=3;
            }
        }

    }
    else
    {
        //Lost Object from sight: impose hovering
        *distanceInCmParameter=DEFAULTDISTANCEINCM;
        *pixelsFromCenterParameter=0;
        lookaround=1;
    }
}


C_RESULT output_gtk_stage_transform( void *cfg, vp_api_io_data_t *in, vp_api_io_data_t *out)
{
    vp_os_mutex_lock(&video_update_lock);

    /* Get a reference to the last decoded picture */
    pixbuf_data_real      = (uint8_t*)in->buffers[0];

    distanceFromTargetInCm(pixbuf_data_real, &distanceInCmParameter, &pixelsFromCenterParameter, &chancheOfShootParameter);

    vp_os_mutex_unlock(&video_update_lock);

    return (SUCCESS);
}

C_RESULT output_gtk_stage_close( void *cfg, vp_api_io_data_t *in, vp_api_io_data_t *out)
{
    return (SUCCESS);
}


const vp_api_stage_funcs_t vp_stages_output_gtk_funcs =
{
    NULL,
    (vp_api_stage_open_t)output_gtk_stage_open,
    (vp_api_stage_transform_t)output_gtk_stage_transform,
    (vp_api_stage_close_t)output_gtk_stage_close
};

DEFINE_THREAD_ROUTINE(video_stage, data)
{
    C_RESULT res;

    vp_api_io_pipeline_t    pipeline;
    vp_api_io_data_t        out;
    vp_api_io_stage_t       stages[NB_STAGES];

    vp_api_picture_t picture;

    video_com_config_t              icc;
    vlib_stage_decoding_config_t    vec;
    vp_stages_yuv2rgb_config_t      yuv2rgbconf;
#ifdef RECORD_VIDEO
    video_stage_recorder_config_t   vrc;
#endif
    /// Picture configuration
    picture.format        = PIX_FMT_YUV420P;

    picture.width         = QVGA_WIDTH;
    picture.height        = QVGA_HEIGHT;
    picture.framerate     = 30;

    picture.y_buf   = vp_os_malloc( QVGA_WIDTH * QVGA_HEIGHT     );
    picture.cr_buf  = vp_os_malloc( QVGA_WIDTH * QVGA_HEIGHT / 4 );
    picture.cb_buf  = vp_os_malloc( QVGA_WIDTH * QVGA_HEIGHT / 4 );

    picture.y_line_size   = QVGA_WIDTH;
    picture.cb_line_size  = QVGA_WIDTH / 2;
    picture.cr_line_size  = QVGA_WIDTH / 2;

    vp_os_memset(&icc,          0, sizeof( icc ));
    vp_os_memset(&vec,          0, sizeof( vec ));
    vp_os_memset(&yuv2rgbconf,  0, sizeof( yuv2rgbconf ));

    icc.com                 = COM_VIDEO();
    icc.buffer_size         = 100000;
    icc.protocol            = VP_COM_UDP;
    COM_CONFIG_SOCKET_VIDEO(&icc.socket, VP_COM_CLIENT, VIDEO_PORT, wifi_ardrone_ip);

    vec.width               = QVGA_WIDTH;
    vec.height              = QVGA_HEIGHT;
    vec.picture             = &picture;
    vec.block_mode_enable   = TRUE;
    vec.luma_only           = FALSE;

    yuv2rgbconf.rgb_format = VP_STAGES_RGB_FORMAT_RGB24;
#ifdef RECORD_VIDEO
    vrc.fp = NULL;
#endif

    pipeline.nb_stages = 0;

    stages[pipeline.nb_stages].type    = VP_API_INPUT_SOCKET;
    stages[pipeline.nb_stages].cfg     = (void *)&icc;
    stages[pipeline.nb_stages].funcs   = video_com_funcs;

    pipeline.nb_stages++;

#ifdef RECORD_VIDEO
    stages[pipeline.nb_stages].type    = VP_API_FILTER_DECODER;
    stages[pipeline.nb_stages].cfg     = (void*)&vrc;
    stages[pipeline.nb_stages].funcs   = video_recorder_funcs;

    pipeline.nb_stages++;
#endif // RECORD_VIDEO
    stages[pipeline.nb_stages].type    = VP_API_FILTER_DECODER;
    stages[pipeline.nb_stages].cfg     = (void*)&vec;
    stages[pipeline.nb_stages].funcs   = vlib_decoding_funcs;

    pipeline.nb_stages++;

    stages[pipeline.nb_stages].type    = VP_API_FILTER_YUV2RGB;
    stages[pipeline.nb_stages].cfg     = (void*)&yuv2rgbconf;
    stages[pipeline.nb_stages].funcs   = vp_stages_yuv2rgb_funcs;

    pipeline.nb_stages++;

    stages[pipeline.nb_stages].type    = VP_API_OUTPUT_SDL;
    stages[pipeline.nb_stages].cfg     = NULL;
    stages[pipeline.nb_stages].funcs   = vp_stages_output_gtk_funcs;

    pipeline.nb_stages++;

    pipeline.stages = &stages[0];

    /* Processing of a pipeline */
    if( !ardrone_tool_exit() )
    {
        PRINT("\n   Video stage thread initialisation\n\n");

        res = vp_api_open(&pipeline, &pipeline_handle);

        if( SUCCEED(res) )
        {
            int loop = SUCCESS;
            out.status = VP_API_STATUS_PROCESSING;

            while( !ardrone_tool_exit() && (loop == SUCCESS) )
            {
                if( SUCCEED(vp_api_run(&pipeline, &out)) )
                {
                    if( (out.status == VP_API_STATUS_PROCESSING || out.status == VP_API_STATUS_STILL_RUNNING) )
                    {
                        loop = SUCCESS;
                    }
                }
                else loop = -1; // Finish this thread
            }

            vp_api_close(&pipeline, &pipeline_handle);
        }
    }

    PRINT("   Video stage thread ended\n\n");

    return (THREAD_RET)0;
}



