/**
 * @file main.c
 * @author sylvain.gaeremynck@parrot.com
 * @date 2009/07/01
 */
#include <ardrone_testing_tool.h>

//ARDroneLib
#include <ardrone_tool/ardrone_time.h>
#include <ardrone_tool/Navdata/ardrone_navdata_client.h>
#include <ardrone_tool/Control/ardrone_control.h>
#include <ardrone_tool/UI/ardrone_input.h>

//Common
#include <config.h>
#include <ardrone_api.h>

//VP_SDK
#include <ATcodec/ATcodec_api.h>
#include <VP_Os/vp_os_print.h>
#include <VP_Api/vp_api_thread_helper.h>
#include <VP_Os/vp_os_signal.h>

//Local project
#include <UI/gamepad.h>
#include <Video/video_stage.h>

//gtk support
#include <gtk/gtk.h>

//config support
#include <libconfig.h>

#include <global_variables.h>

static int32_t exit_ihm_program = 1;

/* Implementing Custom methods for the main function of an ARDrone application */

/* The delegate object calls this method during initialization of an ARDrone application */
C_RESULT ardrone_tool_init_custom(int argc, char **argv)
{

    double dconftemp=0;
    long int lconftemp=0;

    config_t config;

    /* Registering for a new device of game controller */
    ardrone_tool_input_add( &gamepad );

    /*memsetting navdata at zero*/

    vp_os_mutex_lock(&navdata_update_lock);
    vp_os_memset(&user_navdata,0,sizeof(navdata_unpacked_t));
    vp_os_mutex_unlock(&navdata_update_lock);

    /*allocates memory for images*/

    vp_os_mutex_lock(&video_update_lock);

    pixbuf_data_robe=vp_os_malloc(320*240*3);
    pixbuf_data_sword=vp_os_malloc(320*240*3);

    vp_os_mutex_unlock(&video_update_lock);

    /*parsing both config file for settings*/

    config_init (&config);

    printf("vc read %d\n",config_read_file (&config,"videoconfig.txt"));

    config_lookup_int (&config, "MINHROBE", &lconftemp);
    MINHROBE= lconftemp;
    config_lookup_int (&config, "MAXHROBE", &lconftemp);
    MAXHROBE= lconftemp;
    config_lookup_int (&config, "MINSROBE", &lconftemp);
    MINSROBE= lconftemp;
    config_lookup_int (&config, "MAXSROBE", &lconftemp);
    MAXSROBE= lconftemp;
    config_lookup_int (&config, "MINVROBE", &lconftemp);
    MINVROBE= lconftemp;
    config_lookup_int (&config, "MAXVROBE", &lconftemp);
    MAXVROBE= lconftemp;

    config_lookup_int (&config, "MINHSWORDL", &lconftemp);
    MINHSWORDL= lconftemp;
    config_lookup_int (&config, "MAXHSWORDL", &lconftemp);
    MAXHSWORDL= lconftemp;
    config_lookup_int (&config, "MINHSWORDH", &lconftemp);
    MINHSWORDH= lconftemp;
    config_lookup_int (&config, "MAXHSWORDH", &lconftemp);
    MAXHSWORDH= lconftemp;
    config_lookup_int (&config, "MINSSWORD", &lconftemp);
    MINSSWORD= lconftemp;
    config_lookup_int (&config, "MAXSSWORD", &lconftemp);
    MAXSSWORD= lconftemp;
    config_lookup_int (&config, "MINVSWORD", &lconftemp);
    MINVSWORD= lconftemp;
    config_lookup_int (&config, "MAXVSWORD", &lconftemp);
    MAXVSWORD= lconftemp;

    config_destroy (&config);

    config_init (&config);

    printf("c read %d\n",config_read_file (&config,"config.txt"));

    config_lookup_int (&config,"ERRORFROMCENTER",&lconftemp);
    ERRORFROMCENTER= lconftemp;
    config_lookup_int (&config,"ERRORDISTANCE",&lconftemp);
    ERRORDISTANCE= lconftemp;
    config_lookup_int (&config,"ERRORHEIGHT",&lconftemp);
    ERRORHEIGHT= lconftemp;
    config_lookup_float (&config,"YAWK",&dconftemp);
    YAWK=dconftemp;
    config_lookup_float (&config,"GAZK",&dconftemp);
    GAZK=dconftemp;
    config_lookup_int (&config,"DEFAULTHEIGHT",&lconftemp);
    DEFAULTHEIGHT= lconftemp;
    config_lookup_float (&config,"PASTWEIGHT_PITCH",&dconftemp);
    PASTWEIGHT_PITCH=dconftemp;
    config_lookup_int (&config,"DEFAULTDISTANCEINCM",&lconftemp);
    DEFAULTDISTANCEINCM= lconftemp;
    config_lookup_int (&config,"SHOTEVALUATIONREQUESTVALUE",&lconftemp);
    SHOTEVALUATIONREQUESTVALUE= lconftemp;
    config_lookup_int (&config,"buffWidth",&lconftemp);
    buffWidth= lconftemp;
    config_lookup_int (&config,"buffHeight",&lconftemp);
    buffHeight= lconftemp;
    config_lookup_int (&config,"itemHeightInCm",&lconftemp);
    itemHeightInCm= lconftemp;
    config_lookup_int (&config,"minAllowedPixelHeight",&lconftemp);
    minAllowedPixelHeight= lconftemp;
    config_lookup_int (&config,"pointOfFocus",&lconftemp);
    pointOfFocus= lconftemp;
    config_lookup_float (&config,"PASTWEIGHT_DIST",&dconftemp);
    PASTWEIGHT_DIST=dconftemp;
    config_lookup_float (&config,"maxChancheOfShoot",&dconftemp);
    maxChancheOfShoot=dconftemp;
    config_lookup_float (&config,"DISTLOWKPLUS",&dconftemp);
    DISTLOWKPLUS=dconftemp;
    config_lookup_float (&config,"DISTHIGHKPLUS",&dconftemp);
    DISTHIGHKPLUS=dconftemp;
    config_lookup_float (&config,"DISTZEROLOWPOINTPLUS",&dconftemp);
    DISTZEROLOWPOINTPLUS=dconftemp;
    config_lookup_float (&config,"DISTLOWHIGHPOINTPLUS",&dconftemp);
    DISTLOWHIGHPOINTPLUS=dconftemp;
    config_lookup_float (&config,"DISTLOWKMINUS",&dconftemp);
    DISTLOWKMINUS=dconftemp;
    config_lookup_float (&config,"DISTHIGHKMINUS",&dconftemp);
    DISTHIGHKMINUS=dconftemp;
    config_lookup_float (&config,"DISTZEROLOWPOINTMINUS",&dconftemp);
    DISTZEROLOWPOINTMINUS=dconftemp;
    config_lookup_float (&config,"DISTLOWHIGHPOINTMINUS",&dconftemp);
    DISTLOWHIGHPOINTMINUS=dconftemp;

    config_destroy (&config);

    /* Start all threads of your application */
    START_THREAD( video_stage, NULL );
    START_THREAD( userinterface, NULL );
    START_THREAD( chasing, NULL );
    START_THREAD( intelligence, NULL );
    START_THREAD( atmanager, NULL );

    return C_OK;
}

/* The delegate object calls this method when the event loop exit */
C_RESULT ardrone_tool_shutdown_custom()
{
    /* Relinquish all threads of your application */
    JOIN_THREAD( video_stage );
    JOIN_THREAD( userinterface );
    JOIN_THREAD( chasing );
    JOIN_THREAD( intelligence );
    JOIN_THREAD( atmanager );

    /* Unregistering for the current device */
    ardrone_tool_input_remove( &gamepad );

    return C_OK;
}

/* The event loop calls this method for the exit condition */
bool_t ardrone_tool_exit()
{
    return exit_ihm_program == 0;
}

C_RESULT signal_exit()
{
    exit_ihm_program = 0;

    return C_OK;
}

#include <gtk_callbacks.h>
#include <gtk_thread.h>
#include <fuzzy_controller.h>

/*Chasing Thread*/


DEFINE_THREAD_ROUTINE(chasing, data)
{
    while(1)
    {
        if(chasing_active)
        {
            if(abs(DEFAULTDISTANCEINCM-distanceInCmParameter)>ERRORDISTANCE)
            {
                /* positive roll pushes the drone back
                if we are closer than 180 cm 180-160=20 and we will go backward*/

                userroll=(DEFAULTDISTANCEINCM-distanceInCmParameter)*fuzzy_controller_roll(DEFAULTDISTANCEINCM-distanceInCmParameter);
                if (userroll>1.0)
                    userroll=1.0;
                if (userroll<-1.0)
                    userroll=-1.0;
            }
            else
            {
                userroll=0;
            }
            if (lookaround==0)
            {
                if(abs(pixelsFromCenterParameter)>ERRORFROMCENTER)
                {

                    /* positive yaw makes the drone spin clockwise
                    if we are on the right (positive value) we will spin right*/

                    useryaw=(pixelsFromCenterParameter)*YAWK;
                    if (useryaw>1.0)
                        useryaw=1.0;
                    if (useryaw<-1.0)
                        useryaw=-1.0;
                }
                else
                {
                    useryaw=0;
                }
            }
            else
            {
                /*Lookaround is set, start turning to look for player*/
                useryaw=0.3;
            }
            if(abs(DEFAULTHEIGHT - user_navdata.navdata_demo.altitude)>ERRORHEIGHT)
            {
                /*keep floating at DEFAULTHEIGHT. Positive gaz pushes you upward*/
                usergaz=(DEFAULTHEIGHT-user_navdata.navdata_demo.altitude)*GAZK;
            }
            else
            {
                usergaz=0;
            }
            usleep(2000);
        }
        else
        {
            /*zeroing everything it's not really needed but we still do it*/
            userroll=0;
            useryaw=0;
            usergaz=0;
            sleep(2);
        }
    }
    return C_OK;
}

DEFINE_THREAD_ROUTINE(intelligence, data)
{
    //maximum side speed the drone will reach
    const float side_speed=0.1;

    /*temporary variables that holds the current pitch while it stops to shoot
    not really useful because of inertia but still worth trying*/
    float speed_store=0;

    /*variables that holds the speed the drone tries to reach
    we don't use directly side_speed because that should be constant*/
    float speed_to_reach=0;

    /*countswitch holds the counter to switching direction for the pitch
    switchchance holds the possible values that are assigned to countswitch
    the values are calculated seconds/0.015, for examples 733*0.015=11 sec*/
    int countswitch=0;
    int switchchanche[]= {   533,533,
                             633,633,633,633,633,
                             733,733,733,733,733,733,733,
                             833,833,833,833,833,
                             933,933
                         };

    //actual chanche of shoot calulated using the uncover percentage
    float actualChancheOfShoot=0;

    //picked random for choosing wether to shoot
    float pickedRandom=0;

    //picked random for choosing how long to keep direction
    int pickedRandomSide=0;

    srand(time(0));

    while(1)
    {
        if(intel_active)
        {
            if(lookaround==0)
            {
                /*if we just entered or reentered after a lookaround
                we need to set the speed to reach*/
                if(speed_to_reach==0)
                    speed_to_reach=side_speed;

                //choose wether to shoot

                actualChancheOfShoot = maxChancheOfShoot * chancheOfShootParameter;

                pickedRandom = rand()/(float)RAND_MAX;

                if (pickedRandom<=actualChancheOfShoot)
                {
                    //SHOOT
                    printf("\a");
                    printf("shoot\n");
                    fflush(stdout);

                    /*Store the side speed, set the pitch at zero and set shotevaluationrequest
                    for the video processing thread. The ansewer will arrive after SHOTEVALUATIONREQUESTVALUE
                    frames are processed. We play the animation and then wait*/
                    speed_store=userpitch;
                    userpitch=0;
                    shotevaluationrequest=SHOTEVALUATIONREQUESTVALUE;
                    ardrone_at_set_anim(8,MAYDAY_TIMEOUT[8]);
                    while(shotevaluationresponse==PENDING)
                        usleep(500);
                    if(shotevaluationresponse==SHOT_PARRIED)
                    {
                        printf("shoot parried\n");
                        fflush(stdout);
                        user_score++;
                    }

                    if(shotevaluationresponse==SHOT_PASSED)
                    {
                        printf("shoot passed\n");
                        fflush(stdout);
                        drone_score++;
                    }

                    /*reset the shotevaluationresponse to PENDING
                    ready for the next shot*/
                    shotevaluationresponse=PENDING;
                    userpitch=speed_store;


                }

                /*Decrement countswitch if set, otherwise pick a new countswitch
                randomly, and change the direction. Pute the pitch over speed_to_reach
                to make the turn sharper, it will gradually decrement later.*/
                if(countswitch)
                {
                    countswitch--;
                }
                else
                {
                    pickedRandomSide = rand()%21;
                    countswitch=switchchanche[pickedRandomSide];
                    speed_to_reach=-speed_to_reach;
                    userpitch=speed_to_reach*1.5;
                    fflush(stdout);
                }

                //update userpitch with the weighted mean
                userpitch=userpitch*PASTWEIGHT_PITCH + speed_to_reach*(1 - PASTWEIGHT_PITCH);
            }
            else
            {
                userpitch=0;
            }

            usleep(15000);


        }
        else
        {
            /*If we are looking around we should not pitch*/
            speed_to_reach=0;//not really necessary
            userpitch=0;
            sleep(2);
        }
    }

    return C_OK;
}

DEFINE_THREAD_ROUTINE(atmanager, data)
{

    while(1)
    {

        if(chasing_active||intel_active)
        {
            //if everything is zeroed put enable to 0 for hovering, else to 1
            if(!(userpitch||userroll))
            {
                userenable=0;
            }
            else
            {
                userenable=1;
            }
            ardrone_at_set_progress_cmd(userenable,userpitch,userroll,usergaz,useryaw);
            usleep(1000);
        }
        else
        {
            ardrone_at_set_progress_cmd(0,0.0,0.0,0.0,0.0);
            sleep(1);
        }
    }

    return C_OK;
}





/* Implementing thread table in which you add routines of your application and those provided by the SDK */
BEGIN_THREAD_TABLE
THREAD_TABLE_ENTRY( ardrone_control, 20 )
THREAD_TABLE_ENTRY( navdata_update, 20 )
THREAD_TABLE_ENTRY( userinterface, 20 )
THREAD_TABLE_ENTRY( chasing, 20 )
THREAD_TABLE_ENTRY( intelligence, 20 )
THREAD_TABLE_ENTRY( atmanager, 20 )
THREAD_TABLE_ENTRY( video_stage, 20 )
END_THREAD_TABLE






